/* This file converts ECE 498 output back to PCX */

#include <stdio.h>  /* For FILE I/O */
#include <string.h> /* For strncmp */
#include <fcntl.h>  /* for open()  */
#include <unistd.h> /* for lseek() */
#include <sys/stat.h> /* for file modes */

static unsigned char high_byte(int value) {
   return (value>>8)&0xff;
}

static unsigned char low_byte(int value) {
   return (value)&0xff;
}

int vmwSavePCX(int *buffer,int xsize,int ysize, int pcx_fd) {
   
    int x,y,oldcolor,color,numacross,plane=0;
    unsigned char *pcx_header;
    unsigned char temp_byte;
          
    //    pcx_fd=open(filename,O_WRONLY|O_CREAT|O_TRUNC,S_IRUSR|S_IWUSR);
    //if (pcx_fd<0) {
    // printf("Error opening \"%s\" for writing!\n",filename);
    // return -1;
    //}
   
    pcx_header=calloc(1,128);

    pcx_header[0]=0x0a;  /* Manufacturer ID-- A=ZSoft .pcx */
    pcx_header[1]=0x05;  /* Version # */
    pcx_header[2]=0x01;  /* Encoding.  1=RLE */
    pcx_header[3]=0x08;  /* Bits Per Pixel */
   
    pcx_header[4]=low_byte(0);  /* Xmin */
    pcx_header[5]=high_byte(0);
   
    pcx_header[6]=low_byte(0);  /* Ymin */
    pcx_header[7]=high_byte(0);   
   
    pcx_header[8]=low_byte(xsize-1);  /* Xmax */
    pcx_header[9]=high_byte(xsize-1);
   
    pcx_header[10]=low_byte(ysize-1); /* Ymax */
    pcx_header[11]=high_byte(ysize-1); /* Ymax */
   
   
    pcx_header[12]=0x2c; /* Horizontal DPI */
    pcx_header[13]=0x01; /* " .. so 0x12c=300dpi */
    pcx_header[14]=0x2c; /* Vertical DPI */
    pcx_header[15]=0x01; /* " .. so 0x12c=300dpi */
   
    pcx_header[65]=0x03; /* Number of color planes */
   
    pcx_header[66]=low_byte(xsize); /* bytes per line */
    pcx_header[67]=high_byte(xsize);
   
    pcx_header[68]=0x01; /* Color Palette */
   
       /* 128 byte PCX Header */
    write(pcx_fd,pcx_header,128);
   
    y=0;
    x=0;
    numacross=0;
   
       /* Set up initial conditions */

    oldcolor=(buffer[0]>>24)&0xff;
    while (y<ysize) {
       color=(buffer[y*xsize+x]>>((3-plane)*8))&0xff;
       if ( (color==oldcolor)&&(numacross<63)&&(x<(xsize-1)) ) numacross++;
       else { /* This pixel not the same color as previous */
	  if ((numacross==1) && (oldcolor<192)) {
	     write(pcx_fd,&oldcolor,1);
	  }
	  else {
	     temp_byte=numacross+192;
	     write(pcx_fd,&temp_byte,1);
	     write(pcx_fd,&oldcolor,1);
             numacross=1;
	  }
       }
       oldcolor=color;
       x++;
       if (x>xsize) {
          x=0;
          numacross=0;
	  plane++;
	  if (plane>2) { 
	     plane=0;
	     y++;
	  }
	  oldcolor=(buffer[y*xsize+x]>>((3-plane)*8))&0xff;

       }
    }
   

    close(pcx_fd);
    return 0;
}

int main(int argc, char **argv) {
   
   int *buffer;
   int xsize,ysize;
   
   //   fff=fopen("output.bin","r");
   //if (fff==NULL) return -1;
   
   fread(&xsize,sizeof(unsigned int),1,stdin);
   fread(&ysize,sizeof(unsigned int),1,stdin);   
   
   buffer=calloc(xsize*ysize,sizeof(unsigned int));
   if (buffer==NULL) return -1;
   
   fread(buffer,sizeof(unsigned int),xsize*ysize,stdin);   
   
   //fclose(fff);
   
   vmwSavePCX(buffer,xsize,ysize,fileno(stdout));
   
   return 0;
}
