/* Converts a 24-bit PCX file to a raw file for use in ECE498 Project 1 */

#include <stdio.h>  /* For FILE I/O */
#include <string.h> /* For strncmp */
#include <fcntl.h>  /* for open()  */
#include <unistd.h> /* for lseek() */
#include <sys/stat.h> /* for file modes */
#include <stdlib.h>  /* exit() */

int vmwLoadPCX(int pcx_fd, unsigned int **output) {
   
   int debug=0,bpp;
   int x,y;
   int i,numacross,offset;
   int xsize,ysize,plane;
   int xmin,ymin,xmax,ymax,version;
   unsigned char pcx_header[128];
   unsigned char temp_byte;
   
       /* Open the file */                  
   //    pcx_fd=open(filename,O_RDONLY);
    
   //if (pcx_fd<0) {
   //    printf("ERROR!  File \"%s\" not found!\n",filename);
   //    return -1;
   // }
   
    /*************** DECODE THE HEADER *************************/
      
    //    lseek(pcx_fd,0,SEEK_SET);
    
    read(pcx_fd,&pcx_header,128);
   
    xmin=(pcx_header[5]<<8)+pcx_header[4];
    ymin=(pcx_header[7]<<8)+pcx_header[6];
   
    xmax=(pcx_header[9]<<8)+pcx_header[8];
    ymax=(pcx_header[11]<<8)+pcx_header[10];

    version=pcx_header[1];
    bpp=pcx_header[3];
   
    if (debug) {
	
       fprintf(stderr,"Manufacturer: ");
       if (pcx_header[0]==10) fprintf(stderr,"Zsoft\n");
       else fprintf(stderr,"Unknown %i\n",pcx_header[0]);
   
       fprintf(stderr,"Version: ");

       switch(version) {
        case 0: fprintf(stderr,"2.5\n"); break;
        case 2: fprintf(stderr,"2.8 w palette\n"); break;
        case 3: fprintf(stderr,"2.8 w/o palette\n"); break;
        case 4: fprintf(stderr,"Paintbrush for Windows\n"); break;
        case 5: fprintf(stderr,"3.0+\n"); break;
        default: fprintf(stderr,"Unknown %i\n",version);
       }
       fprintf(stderr,"Encoding: ");
       if (pcx_header[2]==1) fprintf(stderr,"RLE\n");
       else fprintf(stderr,"Unknown %i\n",pcx_header[2]);
       
       fprintf(stderr,"BitsPerPixelPerPlane: %i\n",bpp);
       fprintf(stderr,"File goes from %i,%i to %i,%i\n",xmin,ymin,xmax,ymax);
   
       fprintf(stderr,"Horizontal DPI: %i\n",(pcx_header[13]<<8)+pcx_header[12]);
       fprintf(stderr,"Vertical   DPI: %i\n",(pcx_header[15]<<8)+pcx_header[14]);
       
       fprintf(stderr,"Number of colored planes: %i\n",pcx_header[65]);
       fprintf(stderr,"Bytes per line: %i\n",(pcx_header[67]<<8)+pcx_header[66]);
       fprintf(stderr,"Palette Type: %i\n",(pcx_header[69]<<8)+pcx_header[68]);
       fprintf(stderr,"Hscreen Size: %i\n",(pcx_header[71]<<8)+pcx_header[70]);
       fprintf(stderr,"Vscreen Size: %i\n",(pcx_header[73]<<8)+pcx_header[72]);
       
    }

//    if ((version==5) && (bpp==8) && (pcx_header[65]==3)) type=24;
//    else if (version==5) type=8;
//    else type=0;

    xsize=((xmax-xmin)+1);
    ysize=((ymax-ymin)+1);
       
    *output=calloc((xsize*ysize+2),sizeof(unsigned int));
    if (*output==NULL) return -1;

    (*output)[0]=xsize;
    (*output)[1]=ysize;
   offset=2;
       
   x=0; y=0;
  
   int rgb[320][3];
      
   while(y<ysize) {   
      for(plane=0;plane<3;plane++) {
      x=0;	 
       while (x<xsize) {
          read(pcx_fd,&temp_byte,1);
          if (0xc0 == (temp_byte&0xc0)) {
	     numacross=temp_byte&0x3f;
	     read(pcx_fd,&temp_byte,1);
//	     fprintf(stderr,"%i pixels of %i\n",numacross,temp_byte);
	     for(i=0;i<numacross;i++) {
		rgb[x][plane]=temp_byte;
		x++;
	     }
          }
          else {
//	     fprintf(stderr,"%i, %i Color=%i\n",x,y,temp_byte);
	     rgb[x][plane]=temp_byte;
	     x++;
          }
       }

      }
      for(i=0;i<xsize;i++) {
	 (*output)[offset]=(rgb[i][0]<<24 | rgb[i][1]<<16 | rgb[i][2]<<8);
	 offset++;
      }
      y++;
   }
   
   //    close(pcx_fd);

    return 0;
}


int main(int argc, char **argv) {
   
    unsigned int *buffer;
    int result;//,x,y;
    //    FILE *fff;
   
    //char filename[]="butterfinger.pcx";
   
    /* read from stdin */

    result=vmwLoadPCX(fileno(stdin),&buffer);
     
    if (result<0) {
       fprintf(stderr,"Error reading PCX from stdin\n");
       exit(1);
    }
   
    //   fff=fopen("input.bin","w");
    //if (fff==NULL) exit(1);
   
   fwrite(buffer,sizeof(unsigned int),buffer[0]*buffer[1]+2,stdout);
//    fprintf(stderr,"Size: %d x %d\n",buffer[0],buffer[1]);
//   for(y=0;y<buffer[1];y++) {
//    for(x=0;x<buffer[0];x++) {
//       fprintf(stderr,"0x%x\n",buffer[(y*buffer[0])+x+2]);
//    }
//   }
    return 0;
}
